@@ -155,15 +155,18 @@ PREFIX=$(setup_prefix_default "$DEFPREFIX" "$PREFIX")
 SYSPREFIX="$PREFIX/target"
 CROSSPREFIX="$PREFIX/crosstools"
 
-CH_ERROR="$(check_crosser_env $PREFIX $TARGET)"
-if test "x$CH_ERROR" != "x" &&
-   test "x$STEP_CHAIN" != "xyes" &&
-   ( test "x$STEP_BASELIB" = "xyes" ||
-     test "x$STEP_GTK" = "xyes" )
+if test "x$CROSS_OFF" != "xyes"
 then
-  log_error "$CH_ERROR"
-  log_error "Step 'chain' for building environment is not enabled."
-  exit 1
+  CH_ERROR="$(check_crosser_env $PREFIX $TARGET)"
+  if test "x$CH_ERROR" != "x" &&
+     test "x$STEP_CHAIN" != "xyes" &&
+     ( test "x$STEP_BASELIB" = "xyes" ||
+       test "x$STEP_GTK" = "xyes" )
+  then
+    log_error "$CH_ERROR"
+    log_error "Step 'chain' for building environment is not enabled."
+    exit 1
+  fi
 fi
 
 #############################################################################
@@ -274,12 +277,19 @@ build_with_native_compiler() {
 # $2   - Source dir in source hierarchy
 # $3   - Configure options
 # $4   - Make targets
+# [$5] - Destdir
 build_with_cross_compiler() {
-  CONFOPTIONS="--build=$BUILD --host=$TARGET --target=$TARGET $3 --disable-nls"
+  if test "x$3" = "x"
+  then
+    PREFIXOPTION="--prefix=/usr"
+  else
+    PREFIXOPTION=""
+  fi
+  CONFOPTIONS="--build=$BUILD --host=$TARGET --target=$TARGET $PREFIXOPTION $3 --disable-nls"
 
   export CFLAGS="-O2"
-  export CPPFLAGS="-isystem $SYSPREFIX/include"
-  export LDFLAGS="-L$SYSPREFIX/lib"
+  export CPPFLAGS="-isystem $SYSPREFIX/include -isystem $SYSPREFIX/usr/include"
+  export LDFLAGS="-L$SYSPREFIX/lib -L$SYSPREFIX/usr/lib"
 
   if test "x$4" = "x"
   then
@@ -288,7 +298,14 @@ build_with_cross_compiler() {
     MAKETARGETS="$4"
   fi
 
-  if ! build_generic "tgt-$1" "$2" "$CONFOPTIONS" "DESTDIR=$SYSPREFIX $MAKETARGETS"
+  if test "x$5" = "x"
+  then
+    DESTDIR="$SYSPREFIX"
+  else
+    DESTDIR="$5"
+  fi
+
+  if ! build_generic "tgt-$1" "$2" "$CONFOPTIONS" "DESTDIR=$DESTDIR $MAKETARGETS"
   then
     return 1
   fi
@@ -358,9 +375,16 @@ build_glibc() {
 # $3 - Configure options
 # $4 - Make targets
 build_zlib() {
-  export CC=$TARGET-gcc
-  export RANLIB=$TARGET-ranlib
-  export AR=$TARGET-ar
+  if test "x$CROSS_OFF" = "xyes"
+  then
+    export CC=gcc
+    export RANLIB=ranlib
+    export AR=ar
+  else
+    export CC=$TARGET-gcc
+    export RANLIB=$TARGET-ranlib
+    export AR=$TARGET-ar
+  fi
 
   export CFLAGS=""
   export CPPFLAGS="-isystem $SYSPREFIX/include"
@@ -422,8 +446,8 @@ kernel_header_setup() {
   if ! (
     cd $MAINSRCDIR/linux-$VERSION_KERNEL
 
-    if test -f $MAINPACKETDIR/kernel-$TARGET.config ; then
-      cp $MAINPACKETDIR/kernel-$TARGET.config .config
+    if test -f $PACKETDIR/kernel-$TARGET.config ; then
+      cp $PACKETDIR/kernel-$TARGET.config .config
       KCONFTARGET=silentoldconfig
     else
       KCONFTARGET=defconfig
@@ -666,9 +690,15 @@ then
   exit 1
 fi
 
+if ! packetdir_check
+then
+  log_error "Packetdir missing"
+  exit 1
+fi
+
 if test "x$CROSSER_DOWNLOAD" = "xyes"
 then
-  if ! $MAINPACKETDIR/download_packets.sh "$STEPPARAM"
+  if ! ( cd $PACKETDIR && $MAINDIR/scripts/download_packets.sh "$STEPPARAM" )
   then
     log_error "Download failed"
     exit 1
@@ -843,7 +873,9 @@ then
           patch_src glibc-$VERSION_GLIBC/glibc-ports-$VERSION_GLIBC glibc_ports_arm_docargs) ||
        ! (is_minimum_version $VERSION_GLIBC 2.8 ||
           patch_src glibc-$VERSION_GLIBC/glibc-ports-$VERSION_GLIBC glibc_ports_arm_pageh_inc) ||
-       ! patch_src glibc-$VERSION_GLIBC/ports glibc_ports_arm_tlsinc
+       ! patch_src glibc-$VERSION_GLIBC/ports glibc_ports_arm_tlsinc      ||
+       ! (is_smaller_version $VERSION_GLIBC 2.9 ||
+          patch_src glibc-$VERSION_GLIBC/ports glibc_upstream_arm_sigsetjmp)
     then
       crosser_error "Glibc unpacking failed"
       exit 1
@@ -894,6 +926,8 @@ then
 
   echo "Setup:   $TARGET"          >  "$PREFIX/crosser/crosser.hierarchy"
   echo "Version: $CROSSER_VERSION" >> "$PREFIX/crosser/crosser.hierarchy"
+
+  generate_setup_scripts $PREFIX
 else # STEP_CHAIN
   # Set PATH only if it's not already correct to avoid unnecessary hash reset.
   export PATH="$PATH_CROSS"
@@ -902,7 +936,7 @@ fi   # STEP_CHAIN
 
 export CCACHE_DIR="$PREFIX/.ccache"
 
-export PKG_CONFIG_PATH="$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/local/lib/pkgconfig"
+export PKG_CONFIG_PATH="$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
 
 if test "x$STEP_BASELIB" = "xyes"
 then
@@ -951,15 +985,24 @@ then
            ( patch_src glib-$VERSION_GLIB glib_gmoddef &&
              autogen_component glib    $VERSION_GLIB "aclocal automake autoconf" ))      ||
        ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
-         "$GLIB_VARS"                                                                    ||
+         "--prefix=/usr $GLIB_VARS"                                                                    ||
        ! unpack_component          freetype      $VERSION_FREETYPE                       ||
        ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              \
-         "--prefix=/usr"                                                                 ||
+         "--prefix=$PREFIX/interm" "" "/"                                                ||
+       ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              ||
        ! unpack_component          expat         $VERSION_EXPAT                          ||
        ! build_with_cross_compiler expat         expat-$VERSION_EXPAT                    ||
        ! unpack_component          pixman        $VERSION_PIXMAN                         ||
        ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
-         "--disable-gtk"
+         "--prefix=/usr --disable-gtk"                                                   ||
+       ! unpack_component          fontconfig    $VERSION_FONTCONFIG                     ||
+       ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_cross                       ||
+       ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_libtool                     ||
+       ! autogen_component fontconfig $VERSION_FONTCONFIG                                \
+         "libtoolize aclocal automake autoconf"                                          ||
+       ! build_with_cross_compiler fontconfig                                            \
+          fontconfig-$VERSION_FONTCONFIG                                                 \
+          "--prefix=/usr --with-freetype-config=$PREFIX/interm/bin/freetype-config --with-arch=$TARGET"
     then
       crosser_error "gtk+ chain build failed"
       exit 1
@@ -967,6 +1010,12 @@ then
   fi
 fi
 
-generate_setup_scripts $PREFIX
+if test "x$STEP_TEST" = "xyes"
+then
+  STEP="test"
+  STEPADD="    "
+
+  # This is debugging step used only temporarily while testing.
+fi
 
 log_write 1 "SUCCESS"
